<?php
/**
 * Plugin Name: RSS Just Better
 * Plugin URI: http://www.stefaniamarchisio.com/RSS-Just-Better/
 * Description: A list of RSS items given the feed URL and the number of displayable Items. It works as both widget and shortcode.
 * Version: 0.1
 * Author: Stefania Marchisio
 * Author URI: http://stefaniamarchisio.com/about/
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */

/* 
This is a wordpress plugin compatible with wordpress 2.8+ as a widget; wordpress 2.5+ as shortcode.
From the readme.txt:
A list of RSS items given the feed URL and the number of displayable items. It works as both widget and shortcode.

Copyright (C) 2010  Stefania Marchisio (email: mywizardwebs@gmail.com)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

/**
 * Add function to widgets_init
 * @since 0.1
 */
add_action( 'widgets_init', 'funct_RSSjb' );

/**
 * Register our widget.
 * 'WP_RSSjb' is the widget class used below.
 *
 * @since 0.1
 */
function funct_RSSjb() {
   register_widget( 'WP_RSSjb' );
}

/**
 * WP_RSSjb Widget class.
 * This class handles everything that needs to be handled with the widget:
 * the settings, form, display, and update.  Nice!
 *
 * @since 0.1
 */
class WP_RSSjb extends WP_Widget {

	/**
	 * Widget setup.
	 */
	function WP_RSSjb() {
		/* Widget settings. */
		$widget_ops = array( 'classname' => 'rssjustbetter', 'description' => __('A list of RSS items given feed URLs and number of displayable news. It works as both widget and shortcode', 'RSSjb') );

		/* Widget control settings. */
		$control_ops = array( 'width' => 300, 'height' => 350, 'id_base' => 'rss-just-better' );

		/* Create the widget. */
		$this->WP_Widget( 'rss-just-better', __('RSS Just Better', 'RSSjb'), $widget_ops, $control_ops );
	}

	/**
	 * How to display the widget on the screen.
	 */
	function widget( $args, $instance ) {
		extract( $args );

		/* Our variables from the widget settings. */
		$title = apply_filters('widget_title', $instance['title'] );
		$feed = $instance['name'];
		$num = $instance['number'];
		$list = $instance['list'];
		$target= $instance['target'];
		$pubdate = isset( $instance['pubdate'] ) ? $instance['pubdate'] : false;
		$pubtime = isset( $instance['pubtime'] ) ? $instance['pubtime'] : false;
		$pubauthor = isset( $instance['pubauthor'] ) ? $instance['pubauthor'] : false;
		$excerpt = isset( $instance['excerpt'] ) ? $instance['excerpt'] : false;
		$charex = $instance['charex'];

	/* Before widget (defined by themes). */
		echo $before_widget;

		/* Display the widget title if one was input (before and after defined by themes). */
		if ( $title )
			echo $before_title . $title . $after_title;

		/* Call the function to  */
    echo rssjb_FeedList($feed, $num, $list, $target, $pubdate, $pubtime, $pubauthor, $excerpt, $charex);

		/* After widget (defined by themes). */
		echo $after_widget;
	}

	/**
	 * Update the widget settings.
	 */
	function update( $new_instance, $old_instance ) {
		$instance = $old_instance;

		/* Strip tags for title and name to remove HTML (important for text inputs). */
		$instance['title'] = strip_tags($new_instance['title']);
		$instance['name'] = strip_tags($new_instance['name']);
		$instance['number'] = strip_tags($new_instance['number']);
		$instance['charex'] = strip_tags($new_instance['charex']);

		/* No need to strip tags for drop-down menus */
		$instance['target'] = $new_instance['target'];
		$instance['list'] = $new_instance['list'];

		/* No need to strip tags for boolean */
		$instance['pubdate'] = $new_instance['pubdate'];
		$instance['pubtime'] = $new_instance['pubtime'];
		$instance['pubauthor'] = $new_instance['pubauthor'];
		$instance['excerpt'] = $new_instance['excerpt'];

    return $instance;

	}

	/**
	 * Displays the widget settings controls on the widget panel.
	 * Make use of the get_field_id() and get_field_name() function
	 * when creating your form elements. This handles the confusing stuff.
	 */
	function form( $instance ) {

		/* Default widget settings. */
		$defaults = array( 'title' => __('The Latest RSS Items', 'RSSjb'), 'name' => __('http://feeds.feedburner.com/StefaniasBlog', 'RSSjb'), 'number' => '10', 'pubdate' => 'off', 'pubtime' => 'off', 'pubauthor' => 'on', 'excerpt' => 'off', 'charex' => '');

		$instance = wp_parse_args( (array) $instance, $defaults ); 
?>
	<!-- Widget Title: Text Input -->
	<p>
		<label for="<?php echo $this->get_field_id( 'title' ); ?>"><?php _e('Widget Title:', 'RSSjb'); ?></label>
		<input id="<?php echo $this->get_field_id( 'title' ); ?>" name="<?php echo $this->get_field_name( 'title' ); ?>" value="<?php echo $instance['title']; ?>" style="width:100%;" />
	</p>

	<!-- RSS feed: Text Input -->
	<p>
		<label for="<?php echo $this->get_field_id( 'name' ); ?>"><?php _e('RSS Feed:', 'RSSjb'); ?></label>
		<input id="<?php echo $this->get_field_id( 'name' ); ?>" name="<?php echo $this->get_field_name( 'name' ); ?>" value="<?php echo $instance['name']; ?>" style="width:100%;" />
	</p>

	<!-- N. articles to display: Text Input -->
	<p>
		<label for="<?php echo $this->get_field_id( 'number' ); ?>"><?php _e('Num. of News:', 'RSSjb'); ?></label>
		<input id="<?php echo $this->get_field_id( 'number' ); ?>" name="<?php echo $this->get_field_name( 'number' ); ?>" value="<?php echo $instance['number']; ?>" style="width:10%;" />
	</p>

	<!-- Show Publication Date? Checkbox -->
	<p>
		<label for="<?php echo $this->get_field_id( 'pubdate' ); ?>"><?php _e('Show publication date?', 'RSSjb'); ?></label>
		<input class="checkbox" type="checkbox" <?php checked( $instance['pubdate'], 'on' ); ?> id="<?php echo $this->get_field_id( 'pubdate' ); ?>" name="<?php echo $this->get_field_name( 'pubdate' ); ?>" /> 
	</p>

	<!-- Show Publication Time? Checkbox -->
	<p>
		<label for="<?php echo $this->get_field_id( 'pubtime' ); ?>"><?php _e('Show publication time?', 'RSSjb'); ?></label>
		<input class="checkbox" type="checkbox" <?php checked( $instance['pubtime'], 'on' ); ?> id="<?php echo $this->get_field_id( 'pubtime' ); ?>" name="<?php echo $this->get_field_name( 'pubtime' ); ?>" /> 
	</p>

	<!-- Show Excerpt? Checkbox -->
	<p>
	<label for="<?php echo $this->get_field_id( 'excerpt' ); ?>"><?php _e('Show excerpt?', 'RSSjb'); ?></label>
	<input class="checkbox" type="checkbox" <?php checked( $instance['excerpt'], 'on' ); ?> id="<?php echo $this->get_field_id( 'excerpt' ); ?>" name="<?php echo $this->get_field_name( 'excerpt' ); ?>" />
 <label for="<?php echo $this->get_field_id( 'charex' ); ?>"><?php _e(' and limit it to ', 'RSSjb'); ?></label>
	<input id="<?php echo $this->get_field_id( 'charex' ); ?>" name="<?php echo $this->get_field_name( 'charex' ); ?>" value="<?php echo $instance['charex']; ?>" style="width:10%;" /> chars.
	<br /><span style="font-size: 0.8em;">(Warning. the excerpt may contain formatting/images: might not be suitable for sidebars)</span>
         </p>

	<!-- List Type: Select Box -->
	<p>
		<label for="<?php echo $this->get_field_id( 'list' ); ?>"><?php _e('List Type:', 'RSSjb'); ?></label> 
		<select id="<?php echo $this->get_field_id( 'list' ); ?>" name="<?php echo $this->get_field_name( 'list' ); ?>" class="widefat" style="width:100%;">
<option value="UL" <?php if ('UL' == $instance['list']) echo 'selected';?>>Unordered (or Dotted) List (default)</option>
<option value="OL" <?php if ('OL' == $instance['list']) echo 'selected';?>>Ordered (or Numbered) List</option>
		</select>
	</p>

	<!-- Target: Select Box -->
	<p>
		<label for="<?php echo $this->get_field_id( 'target' ); ?>"><?php _e('Target:', 'RSSjb'); ?></label> 
		<select id="<?php echo $this->get_field_id( 'target' ); ?>" name="<?php echo $this->get_field_name( 'target' ); ?>" class="widefat" style="width:100%;">
<option value="_blank" <?php if ('_blank' == $instance['target']) echo 'selected';?>>Open link in a new window (default)</option>
<option value="_self" <?php if ('_self' == $instance['target']) echo 'selected';?>>Open link in the same window</option>
		</select>
	</p>

	<!-- Show Author? Checkbox -->
	<p>
		<label for="<?php echo $this->get_field_id( 'pubauthor' ); ?>"><?php _e('Show footer [linkto this author]?', 'RSSjb'); ?></label>
		<input class="checkbox" type="checkbox" <?php checked( $instance['pubauthor'], 'on' ); ?> id="<?php echo $this->get_field_id( 'pubauthor' ); ?>" name="<?php echo $this->get_field_name( 'pubauthor' ); ?>" /> 
<br /><span style="font-size: 0.9em;">(please, say yes)</span>
	</p>

<?php
	}
}

/**
 * The following code is for the shortcode
 * @since 0.2
 */
/**
 * Add shortcode to wordpress
 */
add_shortcode('RSSjb', 'RSSjb_funct');

/**
 * Add shortcode to wordpress
 */
function RSSjb_funct($atts) {
     extract(shortcode_atts(array(
     "feed" => '',
     "num" => '3',
     "list" => 'UL',
     "target" => '_blank',
     "pubdate" => false,
     "pubtime" => false,
     "pubauthor" => true,
     "excerpt" => false,
     "charex" => ''
     ), $atts));

     /* Display name from widget settings if one was input. */
     if ( $feed ) { 
        return rssjb_FeedList($feed, $num, $list, $target, $pubdate, $pubtime, $pubauthor, $excerpt, $charex);
     } else {
        return "Error: No Feed given in the shortcode.";
     }
}

function rssjb_FeedList($feed, $num, $list, $target, $pubdate, $pubtime, $pubauthor, $excerpt, $charex) {
     include_once(ABSPATH . WPINC . '/rss.php');

     if ( $feed != "" && $rss = fetch_rss( $feed ) ) {
        $fList = "<$list>\n";
 	// for ($i = 0; $i < $num; $i++) {
        if ( $num >0 ) { $rss->items = array_slice( $rss->items, 0, $num ); }
	foreach ( (array) $rss->items as $item ) {
	   // $item = $rss->items[$i];
           $fList .= '<li>';
 	   $fList .= '<a target="'. $target . '" href="'.esc_url( $item['link'] ).'">'. $item['title'] .'</a>';
	   if ( $pubdate !== false && $pubdate !== "false") {
	      $fList .= ' - ' . date ( 'd M y', strtotime( $item['pubdate'] ));
           } 

           if ( $pubtime !== false && $pubtime !== "false") {
	      $fList .= ' at ' . date ( 'H:i', strtotime( $item['pubdate'] )); 
           }
           if ( $excerpt != false && $excerpt != "false") {
                if ( $charex > 0 ) { 
		   $fList .= '<br/>'. substr($item['summary'], 0, $charex);
		   $fList .= ' <a target="' . $target . '" href="' . esc_url( $item['link'] ) . '">[...]</a>';
               } else {
		   $fList .= '<br/>'. $item['summary'];
               }
           } 
           $fList .= '</li>';
	 }
        $fList .= "</".$list.">\n";
     } 
		if ($pubauthor !== false && $pubauthor !== "false") {
			$fList .= '<p style="text-align:center; font-size: 0.7em">powered by <a target="_blank" href="http://www.stefaniamarchisio.com/portfolio/">RSS Just Better plugin</a></p>';
    } 
    return $fList;
}

?>
